﻿using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using GlobalBank.EFT.Views;
using GlobalBank.EFT.Tests.Mocks;

namespace GlobalBank.EFT.Tests
{
    /// <summary>
    /// Summary description for LastTransferViewPresenterTestFixture
    /// </summary>
    [TestClass]
    public class LastTransferViewPresenterTestFixture
    {
        public LastTransferViewPresenterTestFixture()
        {
        }

        [TestMethod]
        public void OnLoadSetsAndShowsLastTransferDateInView()
        {
            MockEFTController controller = new MockEFTController();
            controller.TransferDate = new DateTime(2000, 1, 1);
            LastTransferViewPresenter presenter = new LastTransferViewPresenter(controller);
            MockLastTransferView view = new MockLastTransferView();
            presenter.View = view;

            presenter.OnViewLoaded();

            Assert.AreEqual<DateTime>(new DateTime(2000, 1, 1), view.LastTransferDate);
            Assert.IsTrue(view.ShowLastTransferDate);
        }

        [TestMethod]
        public void OnLoadSetsNotShowLastTransferDateInViewIfDateIsMinValue()
        {
            MockEFTController controller = new MockEFTController();
            controller.TransferDate = DateTime.MinValue;
            LastTransferViewPresenter presenter = new LastTransferViewPresenter(controller);
            MockLastTransferView view = new MockLastTransferView();
            view.ShowLastTransferDate = true;
            presenter.View = view;

            presenter.OnViewLoaded();

            Assert.IsFalse(view.ShowLastTransferDate);
        }



        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion
    }

    class MockLastTransferView : ILastTransferView
    {

        private bool _showLastTransferDate = false;
        private DateTime _lastTransferDate;

        #region ILastTransferView Members

        public bool ShowLastTransferDate
        {
            set { _showLastTransferDate = value; }
            get { return _showLastTransferDate; }
        }

        public DateTime LastTransferDate
        {
            set { _lastTransferDate = value; }
            get { return _lastTransferDate; }
        }

        #endregion

    }
}

