//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using Microsoft.Practices.ObjectBuilder;
using GlobalBank.EFT.Interface;
using GlobalBank.EFT.BusinessEntities;
using Microsoft.Practices.CompositeWeb.Web;
using GlobalBank.EFT.PageFlows;
using Microsoft.Practices.PageFlow;
using System.Collections.ObjectModel;

namespace GlobalBank.EFT
{
    public class EFTController
    {
        private IPageFlowProvider _pageflowProvider;
        private IPageFlow _transferPageFlow;

        private ITransferService _transferService;

        public EFTController()
        {
        }

        [InjectionConstructor]
        public EFTController(
          [ServiceDependency] ITransferService transferService
            )
        {
            _transferService = transferService;
        }

        public virtual DateTime GetLastTransferDate()
        {
            return _transferService.GetLastTransferDate();
        }

        public virtual void AddTransfer(Transfer transfer)
        {
            transfer.Id = Guid.NewGuid();
            List<Transfer> transfers = Transfers;
            transfers.Add(transfer);
            Transfers = transfers;
        }

        private List<Transfer> Transfers
        {
            get
            {
                if (!TransferPageFlow.UserData.Contains("Transfers"))
                {
                    Transfers = new List<Transfer>();
                }
                return (List<Transfer>)TransferPageFlow.UserData["Transfers"];
            }
            set
            {
                TransferPageFlow.UserData["Transfers"] = value;
            }
        }

        public virtual ReadOnlyCollection<Transfer> GetTransfers()
        {
            return Transfers.AsReadOnly();
        }

        public virtual void CreateNewTransfers()
        {
            if (TransferPageFlow.Status == PageFlowStatus.NotStarted)
            {
                TransferPageFlow.Start();
            }
            TransferPageFlow.RedirectToCurrentPage();
        }


        public virtual void ReviseTransfers()
        {
            TransferPageFlow.Navigate("ReviseTransfers");
        }

        public virtual void ProcessTransfers()
        {
            Transfer[] transfersProcessed = _transferService.ProcessTransfers(Transfers.ToArray());
            Transfers = new List<Transfer>(transfersProcessed);
            TransferPageFlow.Navigate("ProcessTransfers");
        }

        public virtual void SubmitTransfers()
        {
            TransferPageFlow.Navigate("SubmitTransfer");
        }

        public virtual void FundsTransferComplete()
        {
            TransferPageFlow.Complete();
        }

        [ProviderDependency(typeof(PageFlowDirectory))]
        public IPageFlowProvider PageFlowProvider
        {
            get { return _pageflowProvider; }
            set { _pageflowProvider = value; }
        }

        protected IPageFlow TransferPageFlow
        {
            get
            {
                if (_transferPageFlow == null)
                {
                    _transferPageFlow = _pageflowProvider.GetPageFlow(typeof(TransferPageFlow));
                }
                return _transferPageFlow;
            }
        }

        public virtual void DeleteTransfer(Transfer transfer)
        {
            List<Transfer> transfers = Transfers;

            Transfer found = FindTransfer(transfers, transfer);
            if (found != null)
            {
                transfers.Remove(found);
            }

            Transfers = transfers;
        }

        public virtual void UpdateTransfer(Transfer transfer)
        {
            List<Transfer> transfers = Transfers;
            
            Transfer found = FindTransfer(transfers, transfer);
            if (found != null)
            {
                found.SourceAccount = transfer.SourceAccount;
                found.TargetAccount = transfer.TargetAccount;
                found.Status = transfer.Status;
                found.Amount = transfer.Amount;
            }

            Transfers = transfers;
        }

        private static Transfer FindTransfer(List<Transfer> transfers, Transfer transfer)
        {
            return transfers.Find(delegate(Transfer match)
            {
                return transfer.Id == match.Id;
            });
        }
    }
}
